C+
C     FUPLOT - A set of subroutines to translate calls to the
C              so-called "Basic Plot Package" of Ron Wahl
C              (PLTSET, SCALE, LINE, CHAR, ENDPT, NEATL, XAXIS, YAXIS)
C              to industry-standard CALCOMP-style low level plot calls
C              (PLOTS, PLOT, SYMBOL).
C
C     Link this package between Branch of Geophysics programs (such as
C      CONTOUR) and whatever CALCOMP-style graphics package you have.
C
C     Note: It is recommended that these routines be used only as an
C           expedient way to get Geophysics Branch programs to run
C           with existing plotting packages.  I do not recommend developing
C           new programs using these calls.
C
C     version 1.0    8/6/85     Richard W. Saltus, USGS, Denver, CO 80225
C-
C+
C     PLTSET - Plot system initialization
C ***********************************************************************
C
      Subroutine pltset(iplotr,xboard,yboard,isl)
C
C       iplotr = plotter selected (ignored in this implementation)
C       xboard,yboard = size of plotting surface for selected plotter
C                       currently 100 x 100 inches is returned.  If your
C                       plot is larger than this some programs (e.g. CONTOUR)
C                       may abort - simply increase the returned values in
C                       this subroutine.
C        isl = isl(1) <= 0 do not change plot units
C                      = 1 use inches for plot units
C                      = 2 use centimeters for plot units
C
C     see R. R. Wahl basic plot system documentation for complete argument
C     definitions
C
      Dimension isl(5)
C-
      Common /pltsav/cinch,sdxp,sdyp,sxp,syp,istart,svarea,x0,y0
      Dimension sdxp(2),sdyp(2),sxp(4),syp(4),svarea(2)
C
C     Common definitions:
C
C        cinch = Conversion to inches for plot units
C        sdxp  = Lower (1) and Upper (2) x limits of data area in data units
C        sdyp  = Lower (1) and Upper (2) y limits of data area in data units
C        sxp   = (1) X axis length in inches
C                (2) Axis type 0=linear, <>0=logarithmic (not implemented)
C                (3) Offset of X axis origin from left side of device (inches)
C                (4) Total size of device X dimension
C        syp   = same as sxp, except for Y axis
C        istart= 0 if no line or symbol calls have been made
C              = 1 if line or symbol has already been called
C        svarea= Used by line to save last X,Y plotting position (plot units)
C        x0    = Current X axis origin
C        y0    = Current Y axis origin
C
      istart=0
C
C     if ISL(1) = 0 don't change plot units
C               = 1 then plot units are inches
C               = 2 then plot units are centimeters
C
      xboard=100.
      yboard=100.
      cinch=1.0
      If (isl(1).EQ.2)cinch=2.54
C
      Return
      End
C+
C     SCALE - Plot scaling
C  ****************************************************************
      Subroutine scale(dxp,dyp,xp,yp,nopts,ier)
      Dimension dxp(2),dyp(2),xp(4),yp(4)
C
C     dxp, dyp = Two-element arrays containing limits of plot area in
C                data units:
C       dxp(1) = left X axis limit
C       dxp(2) = right X axis limit
C       dyp(1) = bottom Y axis limit
C       dyp(2) = top Y axis limit
C
C     xp, yp   = Four-element arrays containing plot area size in inches
C        xp(1) = length of X axis
C        xp(2) = not recognized by this package
C        xp(3) = X axis offset from edge of plot (left margin)
C        xp(4) = total X plot dimension (X axis + left margin + right margin)
C        yp(1-4) = same as xp, except for Y axis.
C
C     nopts = size of xp & yp arrays.  May be 1,3, or 4.
C
C     ier   = set to non-zero if scale sizes are in error.
C
C     for complete argument description see R. R. Wahl's Basic plot sys doc.
C-
C
      Common /pltsav/cinch,sdxp,sdyp,sxp,syp,istart,svarea,x0,y0
      Dimension sdxp(2),sdyp(2),sxp(4),syp(4),svarea(2)
C
      ier=0
      sdxp(1)=dxp(1)
      sdxp(2)=dxp(2)
      sdyp(1)=dyp(1)
      sdyp(2)=dyp(2)
C
      If ((nopts.GT.4).OR.(nopts.LT.1)) Then
         ier=1
         Print *,'Bad xp & yp array size in scale call, nopts=',nopts
         Go To 99
         End If
C
      Do 10 i=1,nopts
      sxp(i)=xp(i)/cinch
      syp(i)=yp(i)/cinch
   10 Continue
C
      If (nopts.LE.3) Then
         sxp(4)=sxp(1)+sxp(3)
         syp(4)=syp(1)+syp(3)
         End If
C
C     Other error conditions may be added here for compatibility if desired
C     such as checking for zero-length plot axis, etc.
C
   99 Continue
      Return
      End
C+
C     LINE - Line plotting
C *********************************************************************
      Subroutine line(x,y,n,icon,ipn)
      Dimension x(1),y(1)
C
C     x, y = arrays of length n containing the x and y locations of the
C            lines to be drawn (in data units).
C     n    = size of x & y arrays.
C     icon = if 0 start a new line, if not 0, continue existing line.
C     ipn  = line type (not implemented in this version)
C
C     see R. R. Wahl's Basic plot system documentation for complete
C     argument definitions.
C-
C
      Common /pltsav/cinch,sdxp,sdyp,sxp,syp,istart,svarea,x0,y0
      Dimension sdxp(2),sdyp(2),sxp(4),syp(4),svarea(2)
C
      If (istart.EQ.0)icon=0
      ipen=2
      If (icon.EQ.0)ipen=3
C
C     ckstat will set origin and issue plots call if this is the
C     start of a plot
C
      Call ckstat
C
C     call R. Saltus' mpline subroutine to draw a line with scaling
C     and clipping.
C
      Call mpline2(n,x,y,sdxp(1),sdxp(2),sxp(1),sdyp(1),sdyp(2),syp(1),
     &0,0,0.0,ipen,svarea)
C
      Return
      End
C+
C     VCHAR (char on pre-VAX systems) - Character plotting
C *******************************************************************
C
      Subroutine vchar(x,y,a,n,icode,size,theta,xoff,yoff)
      Dimension x(1),y(1),a(1)
      integer a
C
C     x, y   = center position of character to be plotted plotting
C              (may be arrays for symbol plots)
C     a      = single character or multiple characters in a real array
C     n      = number of characters
C     icode  = type of character plot, 0 for x,y, & a all arrays
C                                      1 for x,y arrays, a single character
C                                      2 for x,y single, a character string
C                                      3 for x,y single in plot units
C                                          and a character string
C     size    = size in inches of characters to be plotted
C     theta   = angle in radians for character direction
C     xoff, yoff = x,y offset from x,y position for character position
C                  in plot units (inches)
C
C     See R. R. Wahl's Basic plot package for complete argument definition
C-
C
      Common /pltsav/cinch,sdxp,sdyp,sxp,syp,istart,svarea,x0,y0
      Dimension sdxp(2),sdyp(2),sxp(4),syp(4),svarea(2)
      Dimension isym(100)
C
      Call ckstat
C
C       convert radians to degrees
C
      angle=(theta/(2*3.14159))*360.
C
C     rotate xoff & yoff
C
      xo=xoff*cos(theta)-yoff*sin(theta)
      yo=xoff*sin(theta)+yoff*cos(theta)
C
C     calculate character shift and rotate it
C       This shift is required because the Wahl character set is defined
C       about the center of the character, whereas the CALCOMP standard
C       is to define the first 13 characters (plot symbols) about the
C       center and all other characters from the lower left corner.
C       
      xst=size/3.5
      yst=size/2.
      xs=xst*cos(theta)-yst*sin(theta)
      ys=xst*sin(theta)+yst*cos(theta)
C
C       disable italics option
C
      ic=mod(icode,4)
C
      If (ic.EQ.0) Then
         Do 10 i=1,n
         Call fixpt(x(i),y(i),xt,yt)
         call upcase(a(i),4,isym)
C
C     if not plotting a centered symbol, center the character being 
C        plotted
C
         if ((isym(1).gt.13).or.(isym(1).le.0)) then
           xt=xt-xs
           yt=yt-ys
           end if
         Call symbol(xt+xo,yt+yo,size,isym,angle,-1)
   10    Continue
         Else If (ic.EQ.1) Then
         call upcase(a,4,isym)
         Do 20 i=1,n
         Call fixpt(x(i),y(i),xt,yt)
C
C     if not plotting a centered symbol, center the character being 
C        plotted
C
         if ((isym(1).gt.13).or.(isym(1).le.0)) then
           xt=xt-xs
           yt=yt-ys
           end if
         Call symbol(xt+xo,yt+yo,size,isym,angle,-1)
   20    Continue
         Else If (ic.EQ.2) Then
C
C        Upshift the character string (for Uppercase only plotters)
C          and make a translation of plotting symbols
C
         Call upcase(a,n,isym)
         Call fixpt(x(1),y(1),xt,yt)
C
C        If n=1 check the single character to see if it is a symbol,
C         otherwise shift xt & yt so that the character is centered
C
         if ((n.gt.1).or.(isym(1).gt.13).or.(isym(1).le.0)) then
           xt=xt-xs
           yt=yt-ys
           end if
         Call symbol(xt+xo,yt+yo,size,isym,angle,n)
         Else If (ic.EQ.3) Then
         xt=x(1)-x0+xo
         yt=y(1)-y0+yo
C
C        Upshift the character string (for Uppercase only plotters)
C
         Call upcase(a,n,isym)
C
C        If n=1 check the single character to see if it is a symbol,
C         otherwise shift xt & yt so that the character is centered
C
         if ((n.gt.1).or.(isym(1).gt.13).or.(isym(1).le.0)) then
           xt=xt-xs
           yt=yt-ys
           end if
         Call symbol(xt,yt,size,isym,angle,n)
         End If
      Return
      End
C+
C     UPCASE - shift alphabetic string to all uppercase
C  *********************************************************
      Subroutine upcase(a,n,isym)
C
C     a = real array containing ascii character information
C     n = number of ascii characters in a
C
C     THIS ROUTINE ASSUMES THAT A SYSTEM WORD CONTAINS 4 CHARACTERS
C     (ie 32 or 36 bit machine)
C-
      Dimension a(1),b(1),isym(1)
      integer a,b
      Character*4 ctemp(1)
      Equivalence (b(1),ctemp(1))
      Dimension itrans(32)
C
C     array itrans is a translation from R. R. Wahl plot symbols to
C       the Calcomp symbols in use in Denver.  These symbols may differ
C       on your system, so a different mapping may be necessary
C
C     The R. R. Wahl definitions for the plot symbols are as follows:
C
C     1 = square    2 = diamond   3 = octogon  4 = X         5 = X with +
C     6 = +         7 = -         8 = Y        9 = X w/ box 10 = hourglass
C    11 = triangle 12 = triangle 13 = small diamond
C
C     The R. R. Wahl definitions for some aux. symbols are:
C
C    14 through 23 are not defined (mapped to blank)
C
C    24 = degree sign   25 = not equal sign  26 = cent sign  27 = right arrow
C    28 = approx. sign  29 = underscore      30 = <=         31 = >=
C
C    32 is blank (space)
C
      data itrans
     & /14,5,1,4,11,3,15,9,10,12,2,6,12*32,22,32,20,31,15,3*32/
C
      iloop=(n+3)/4
      Do 10 i=1,iloop
      b(1)=a(i)
C
C     If your plotter can handle lower case letters, comment out this
C      call to upshift
C
      Call upshift(ctemp(1))
      do 8 j=1,4
      ic=ichar(ctemp(1)(j:j))
      if ((ic.le.32).and.(ic.gt.0)) ctemp(1)(j:j)=char(itrans(ic))
    8 continue
      isym(i)=b(1)
   10 Continue
      End
C+
C     FIXPT - performs scaling from data units to plot units (inches)
C *********************************************************************
C
      Subroutine fixpt(a,b,x,y)
C
C     a,b = position coordinates in data units
C     x,y = position coordinates in plot units (inches)
C-
      Common /pltsav/cinch,sdxp,sdyp,sxp,syp,istart,svarea,x0,y0
      Dimension sdxp(2),sdyp(2),sxp(4),syp(4),svarea(2)
C
      x=(a-sdxp(1))*(sxp(1)/(sdxp(2)-sdxp(1)))
      y=(b-sdyp(1))*(syp(1)/(sdyp(2)-sdyp(1)))
C
      Return
      End
C+
C     CKSTAT - check status on line and vchar calls then
C              initialize plot system (plots call) or change
C              origin (plot call) as required.
C ***********************************************************************
      Subroutine ckstat
C-
C
      Common /pltsav/cinch,sdxp,sdyp,sxp,syp,istart,svarea,x0,y0
      Dimension sdxp(2),sdyp(2),sxp(4),syp(4),svarea(2)
C
      If (istart.EQ.0) Then
         x0=0.0
         y0=0.0
         Call plots(0,0,0)
         istart=istart+1
         End If
C
C      check for new origin
C
      If ((x0.NE.sxp(3)).OR.(y0.NE.syp(3))) Then
         xt=sxp(3)-x0
         yt=syp(3)-y0
         Call plot(xt,yt,-3)
         x0=sxp(3)
         y0=syp(3)
         End If
      Return
      End
C+
C     ENDPT - Plot termination
C
      Subroutine endpt(ie)
      Dimension ie(1)
C
C     ie = an array to receive diagnostic info. about plot
C          this is not implemented in this version.
C
C     see R. R. Wahl's Basic plot system doc. for argument definition
C-
C
      Common /pltsav/cinch,sdxp,sdyp,sxp,syp,istart,svarea,x0,y0
      Dimension sdxp(2),sdyp(2),sxp(4),syp(4),svarea(2)
C
C     call plot to terminate plotting
C
c      Call plot(0.0,0.0,999)
c
c     end frame
c
      call plot(0.,0.,23)
      Return
      End
C+
C     NEATL - plots a "neat" line around the data area
C
      Subroutine neatl
C-
C
      Common /pltsav/cinch,sdxp,sdyp,sxp,syp,istart,svarea,x0,y0
      Dimension sdxp(2),sdyp(2),sxp(4),syp(4),svarea(2)
C
      Call ckstat
      Call plot(0.0,0.0,3)
      Call plot(sxp(1),0.0,2)
      Call plot(sxp(1),syp(1),2)
      Call plot(0.0,syp(1),2)
      Call plot(0.0,0.0,2)
C
      Return
      End
C+
C  R. R. Wahl's XAXIS and YAXIS subroutines
C
C  The following subroutines (XAXIS, YAXIS, and PXZ10) have been lifted
C    directly from the R. R. Wahl "Basic plot package".  See the documentation
C    for that package for argument definitions, etc.
C
C
C  ROUTINE FOR LINEAR X AXIS ANNOTATION.
C
      SUBROUTINE XAXIS(XX,YY,XP,DEL,IP,SIZE,FMT,NFMT)
C-
      INTEGER FMT
      DIMENSION XX(1),YY(1),FMT(1)
C
      COMMON /PXZ99/
     1FSCAL,IPLTR,IPLTX,INCMT,FMTOL,FMTL2,RXYRS,RYXRS,
     2XRASP,YRASP,XRASX,YRASX,XRSPP,YRSPP,IXLOG,IYLOG,
     3RXMIN,RXMAX,RYMIN,RYMAX,XSCLE,YSCLE,XOFFS,YOFFS,
     4IOBUF(128),NCCNT,NCCMX,NCOUT,NCMAX,NWOUT,NCMUL(6),
     5MASKA,MASKS,LUOUT,ICSTR(20),NCSTR,IDECT(10),ISYMN,
     6ISYMP,LZERO,IDWRK(16),NXYRS,NBLKO,
     7IXRAS(50),IYRAS(50),IPENC(50),NXYSI,NCALS,
     8IDASH(8),DASHT(64),DASHL(16),RDASH(8),DASHX(64),
     9NDASH(8),ICHRL,NCHRI,NCHRO,NLINI,NLINO,LUTTY,LUERR,
     1IERCK(90),MAXER,IXAXC,IYAXC,IAXTY,IHYPH,IZROT
C
      IAXTY=1
      IXAXC=1
      IYAXC=2
      CALL PXZ10(XX,YY,XP,DEL,IP,SIZE,FMT,NFMT)
C
      RETURN
C
      END
C
C  ROUTINE FOR LINEAR Y AXIS ANNOTATION.
C
      SUBROUTINE YAXIS(XX,YY,XP,DEL,IP,SIZE,FMT,NFMT)
C
C
      COMMON /PXZ99/
     1FSCAL,IPLTR,IPLTX,INCMT,FMTOL,FMTL2,RXYRS,RYXRS,
     2XRASP,YRASP,XRASX,YRASX,XRSPP,YRSPP,IXLOG,IYLOG,
     3RXMIN,RXMAX,RYMIN,RYMAX,XSCLE,YSCLE,XOFFS,YOFFS,
     4IOBUF(128),NCCNT,NCCMX,NCOUT,NCMAX,NWOUT,NCMUL(6),
     5MASKA,MASKS,LUOUT,ICSTR(20),NCSTR,IDECT(10),ISYMN,
     6ISYMP,LZERO,IDWRK(16),NXYRS,NBLKO,
     7IXRAS(50),IYRAS(50),IPENC(50),NXYSI,NCALS,
     8IDASH(8),DASHT(64),DASHL(16),RDASH(8),DASHX(64),
     9NDASH(8),ICHRL,NCHRI,NCHRO,NLINI,NLINO,LUTTY,LUERR,
     1IERCK(90),MAXER,IXAXC,IYAXC,IAXTY,IHYPH,IZROT
C
      INTEGER FMT
      DIMENSION XX(1),YY(1),FMT(1)
      IAXTY=3
      IXAXC=2
      IYAXC=1
      CALL PXZ10(XX,YY,XP,DEL,IP,SIZE,FMT,NFMT)
C
      RETURN
C
      END
      SUBROUTINE PXZ10(XX,YY,XP,DEL,IP,SIZE,FMT,NFMT)
C
C  GENERAL SUPPORT ROUTINE FOR LINEAR ANNOTATION
C  OF X AND Y AXIS.
C
C	****************************************************
C	NOTE:  THE ORIGNAL WAHL PACKAGE CONTAINED CALLS TO
C	THE USER-DEFINED ROUTINE "CHAR".  THIS CONFLICTS WITH
C	THE VAX VMS FORTRAN INTRINSIC FUNCTION "CHAR".  THEREFORE
C	ALL CALLS TO THE USER-DEFINED ROUTINE HAVE BEEN CHANGED TO
C	"VCHAR."              CINDY COOPER, 23-JUL-81
C	****************************************************
C
      DIMENSION XX(1),YY(1),FMT(1)
      INTEGER A,FMT
C
      COMMON /PXZ99/
     1FSCAL,IPLTR,IPLTX,INCMT,FMTOL,FMTL2,RXYRS,RYXRS,
     2XRASP,YRASP,XRASX,YRASX,XRSPP,YRSPP,IXLOG,IYLOG,
     3RXMIN,RXMAX,RYMIN,RYMAX,XSCLE,YSCLE,XOFFS,YOFFS,
     4IOBUF(128),NCCNT,NCCMX,NCOUT,NCMAX,NWOUT,NCMUL(6),
     5MASKA,MASKS,LUOUT,ICSTR(20),NCSTR,IDECT(10),ISYMN,
     6ISYMP,LZERO,IDWRK(16),NXYRS,NBLKO,
     7IXRAS(50),IYRAS(50),IPENC(50),NXYSI,NCALS,
     8IDASH(8),DASHT(64),DASHL(16),RDASH(8),DASHX(64),
     9NDASH(8),ICHRL,NCHRI,NCHRO,NLINI,NLINO,LUTTY,LUERR,
     1IERCK(90),MAXER,IXAXC,IYAXC,IAXTY,IHYPH,IZROT
C
      LOGICAL NOPS,NOTL
      DIMENSION XY(2),A(5),THTA(4)
      DATA THTA/1.570796,4.71239,0.,3.14159/
      DATA IHYPH/45/
C
      IF (SIZE.LE.0.) GO TO 120
      XY(IYAXC)=YY(1)
      IXYP=MAX0(IABS(IP),1)
      IPSS=0
      THT=THTA(IAXTY)
      IF (XX(1).GT.XX(2)) GO TO 10
      TS=XX(1)
      TE=XX(2)
      GO TO 20
   10 TS=XX(2)
      TE=XX(1)
   20 NOPS=NFMT.LE.0
      NOTL=NOPS.OR.IP.LT.0
      IF (DEL.GT.0.0) GO TO 30
      NTCK=0
      GO TO 40
   30 ISS=TS/DEL
      IE=TE/DEL
      IF (TS.GE.0.0.OR.(TS.LT.0.0.AND.(ISS*DEL).EQ.TS))
     1     ISS=ISS+1
      IF (TE.LT.0.0.OR.(TE.GE.0.0.AND.(IE*DEL).EQ.TE))
     1     IE=IE-1
      NTCK=IE-ISS+1
      DMIN=1.5*SIZE*(TE-TS)/XP
      XYL2=TE-DMIN
   40 CONTINUE
      XOFF=-NFMT*SIZE
      SSB=2.0*SIZE
      XSSB=.333333*SSB
      XSSC=.333333*SIZE
   50 IS=ISS
C
C MIN TICK
C
      XY(IXAXC)=TS
      IF (NOPS) GO TO 60
C	THE BELOW STATEMENT HAS BEEN CHANGED TO COMPLY WITH
C	VAX VMS 2.2 ENCODE FORMAT.    CLC  22-JUL-81
C     ENCODE(A,FMT) TS
      ENCODE(20,FMT,A) TS
      CALL VCHAR(XY(1),XY(2),A,NFMT,2,SIZE,THT,XOFF,0.)
      XYL=TS+DMIN
   60 CALL VCHAR(XY(1),XY(2),IHYPH,1,1,SSB,THT,XSSB ,0.)
      IF (NTCK.LE.0) GO TO 100
C
C INTERMEDIATE TICK LOOP
C
      DO 90 K=1,NTCK
        XY(IXAXC)=IS*DEL
        IF (MOD(IS,IXYP).NE.0) GO TO 70
        S=SSB
        SO=XSSB
        IF (NOTL.OR.XY(IXAXC).LT.XYL.OR.
     1     XY(IXAXC).GT.XYL2) GO TO 80
C	THE BELOW STATEMENT HAS BEEN CHANGED TO COMPLY WITH
C	VAX VMS 2.2 ENCODE FORMAT.    CLC  22-JUL-81
C       ENCODE(A,FMT) XY(IXAXC)
	ENCODE (20,FMT,A) XY(IXAXC)
        CALL VCHAR(XY(1),XY(2),A,NFMT,2,SIZE,THT,XOFF,0.)
        XYL=XY(IXAXC)+DMIN
        GO TO 80
   70   S=SIZE
        SO=XSSC
   80   CALL VCHAR(XY(1),XY(2),IHYPH,1,1,S,THT,SO,0.)
        IS=IS+1
   90   CONTINUE
C
C MAX TICK
C
  100 XY(IXAXC)=TE
      IF (NOPS) GO TO 110
C	THE BELOW STATEMENT HAS BEEN CHANGED TO COMPLY WITH
C	VAX VMS 2.2 ENCODE FORMAT.    CLC  22-JUL-81
C     ENCODE(A,FMT) TE
      ENCODE (20,FMT,A) TE
      CALL VCHAR(XY(1),XY(2),A,NFMT,2,SIZE,THT,XOFF,0.)
  110 CALL VCHAR(XY(1),XY(2),IHYPH,1,1,SSB,THT,XSSB ,0.)
      IF (IPSS.GT.0) GO TO 120
      NOPS=.TRUE.
      IPSS=1
      THT=THTA(IAXTY+1)
      XY(IYAXC)=YY(2)
      GO TO 50
  120 RETURN
      END
C
C+
C
      SUBROUTINE MPLINE2 (N,X,Y,XMIN,XMAX,XL,YMIN,YMAX,YL,
     1                   LINTYP,MARKER,SIZE,IPEN,SVAREA)
C
C MPLINE2 -- Poly-line drawing subroutine
C
C           Draw a line through a set of points, with an optional
C           marker displayed at each point.  (The line may be
C           invisible.)  Clipping is performed using user supplied
C           limits.
C
C CALL MPLINE2(N,X,Y,XMIN,XMAX,XL,YMIN,YMAX,YL,
C             LINTYP,MARKER,SIZE,IPEN,SVAREA)
C
C      N      = Number of data points.
C      X      = X coordinates of data points.
C      Y      = Y coordiantes of data points.
C      XMIN,XMAX = X clipping limits, in data units.
C      XL     = Length of X axis, in plotting units.
C      YMIN,YMAX = Y clipping limits, in data units.
C      YL     = Length of Y axis, in plotting units.
C      LINTYP = Line type flag:  <0 -> Plot markers at points
C                                =0 -> Connect points by lines
C                                >0 -> Connect markers at points with lines
C      MARKER = Symbol number
C      SIZE   = Marker size
C      IPEN   = Move to first point with pen up (IPEN=3) or down (IPEN=2)
C      SVAREA = Real array length 2 to save previous position (in plotting
C               units) for next call to MPLINE2 .
C               SVAREA must be valid if IPEN=2.
C
C Note:  The data value (XMIN,YMIN) is mapped to plotting position (0.,0.);
C        data value (XMAX,YMAX) is mapped to plotting position (XL,YL).
C
C      Called by: User program
C
C          Calls: Plot, Symbol, Plclip
C
C   Commons used: none
C-
C
      DIMENSION  X(1), Y(1), SVAREA(2), VECTOR(4), WINDOW(4)
C
      PX(XV) = (XV-XMIN) / DX
      PY(YV) = (YV-YMIN) / DY
C
      DX = (XMAX-XMIN) / XL
      DY = (YMAX-YMIN) / YL
      WINDOW(1) = 0.0
      WINDOW(2) = 0.0
      WINDOW(3) = XL
      WINDOW(4) = YL
C...  Make sure N >= 1
      IF (N .LE. 0) GOTO 9000
C...  Get previous position from save area
      XP = PX(XMIN)
      YP = PY(YMIN)
      JPEN = 3
      IF (LINTYP .LT. 0) GOTO 1000
      IF (IPEN .NE. 2) GOTO 1000
      XP = SVAREA(1)
      YP = SVAREA(2)
      JPEN = 2
C...  Do loop 1,N
 1000 DO 3000 J = 1,N
         VECTOR(1) = XP
         VECTOR(2) = YP
         VECTOR(3) = PX(X(J))
         VECTOR(4) = PY(Y(J))
         XP = VECTOR(3)
         YP = VECTOR(4)
C...  Clip vector
         CALL PLCLIP(VECTOR,WINDOW,NCLIP)
         IF (NCLIP .GE. 3) GOTO 3000
C...  End points for calls to PLOT and SYMBOL
         XT = VECTOR(3)
         YT = VECTOR(4)
C...  If line, plot line
         IF (LINTYP .LT. 0) GOTO 2000
         CALL PLOT(VECTOR(1),VECTOR(2),3)
         CALL PLOT(XT,YT,JPEN)
         IF (LINTYP .EQ. 0) GOTO 3000
C...  If end point not clipped, plot symbol
 2000    IF (VECTOR(3) .NE. XP) GOTO 3000
         IF (VECTOR(4) .EQ. YP)
     1      CALL SYMBOL(XT,YT,SIZE,MARKER,0.0,-1)
 3000    JPEN = 2
C
C...  Save last point position for next call to MPLINE2.
      SVAREA(1) = XP
      SVAREA(2) = YP
C
C...  Return
 9000 RETURN
C
      END
C+
C     UPSHIFT - Converts lowercase to uppercase.
C               All non-lowercase characters unchanged.
C
C     CALL UPSHIFT(A)
C
C             A = A character string of any length
C
C-
      SUBROUTINE UPSHIFT(A)
      CHARACTER*(*) A,UP*(26),DOWN*(26)
      UP='ABCDEFGHIJKLMNOPQRSTUVWXYZ'
      DOWN='abcdefghijklmnopqrstuvwxyz'
      ILEN=LEN(A)
      DO 10 I=1,ILEN
        INUM=INDEX(DOWN,A(I:I))
        IF (INUM.NE.0) A(I:I)=UP(INUM:INUM)
  10      CONTINUE
      RETURN
      END
C
C+
C
      Subroutine PLCLIP (vector, window, iclip)
C
C PLCLIP - Clip vector to plotting window.
C
C Call PLCLIP (vector,window,iclip)
C
C      vector = The vector to be clipped (in place)
C               (vector(1,1),vector(2,1)) to (vector(1,2),vector(2,2))
C      window = Clipping limits (X-Min,Y-Min), (X-Max,Y-Max)
C      iclip  = 0 If vector good
C             = 1 If vector clipped once
C             = 2 If vector clipped twice
C             = 3 If vector bad
C
C       Called by:  PLPLOT, PLINE1, PLINE2
C
C           Calls:  None
C
C Parameters used:  None
C
C    Commons used:  None
C
C-
C
      Dimension  vector(2,2), window(2,2)
C
C...  Vector limits check and clipping
C
C...  Test each coordinate
      iclip = 0
      Do 100 i = 1,2
         nerr = 0
C
C...     Test individual ordinates
   50    Do 90 j = 1,2
C
C...        Test upper limits
            almt = window(j,2)
C
C...        Upper limit OK?
            If (vector(j,i) .gt. almt) Then
C
C...           Upper limit bad?
               If (vector(j,3-i) .gt. almt) Then
                  Goto 9900
               Else
                  Goto 70
               End If
            End If
C
C...        Test lower limits
            almt = window(j,1)
C
C...        Lower limit OK?
            If (vector(j,i) .lt. almt) Then
C
C...           Lower limit bad?
               If (vector(j,3-i) .lt. almt) Goto 9900
C
C...           Ordinate out of range - attempt intersection
   70          nerr = nerr + 1
C
C...           Intersection failure?
               If (nerr .gt. 2) Then
                  Goto 9900
               Else
                  vector(3-j,i) = ((vector(3-j,1)-vector(3-j,2))/
     1                             (vector(  j,1)-vector(  j,2)))*
     2                             (almt-vector(j,1)) + vector(3-j,1)
                  vector(  j,i) = almt
                  Goto 50
               End If
            End If
   90       Continue
C
C...     Test for any clipping done for this point
  100    If (nerr .gt. 0) iclip = iclip + 1
C
 9000 Return
C
C...  Intersection failure
 9900 iclip = 3
      Goto 9000
C
      End
